@@ -19,28 +19,28 @@
 #' Open Context
 #'
 #' Open a new Context to be used with a Socket. The purpose of a Context is to
-#'     permit applications to share a single socket, with its underlying dialers
-#'     and listeners, while still benefiting from separate state tracking.
+#' permit applications to share a single socket, with its underlying dialers and
+#' listeners, while still benefiting from separate state tracking.
+#'
+#' Contexts allow the independent and concurrent use of stateful operations
+#' using the same socket. For example, two different contexts created on a rep
+#' socket can each receive requests, and send replies to them, without any
+#' regard to or interference with each other.
+#'
+#' Only the following protocols support creation of contexts: req, rep, sub
+#' (in a pub/sub pattern), surveyor, respondent.
+#'
+#' To send and receive over a context use \code{\link{send}} and
+#' \code{\link{recv}} or their async counterparts \code{\link{send_aio}} and
+#' \code{\link{recv_aio}}.
+#'
+#' For nano objects, use the \code{$context_open()} method, which will attach a
+#' new context at \code{$context}. See \code{\link{nano}}.
 #'
 #' @param socket a Socket.
 #'
 #' @return A Context (object of class \sQuote{nanoContext} and \sQuote{nano}).
 #'
-#' @details Contexts allow the independent and concurrent use of stateful
-#'     operations using the same socket. For example, two different contexts
-#'     created on a rep socket can each receive requests, and send replies to
-#'     them, without any regard to or interference with each other.
-#'
-#'     Only the following protocols support creation of contexts: req, rep, sub
-#'     (in a pub/sub pattern), surveyor, respondent.
-#'
-#'     To send and receive over a context use \code{\link{send}} and
-#'     \code{\link{recv}} or their async counterparts \code{\link{send_aio}} and
-#'     \code{\link{recv_aio}}.
-#'
-#'     For nano objects, use the \code{$context_open()} method, which will
-#'     attach a new context at \code{$context}. See \code{\link{nano}}.
-#'
 #' @seealso \code{\link{request}} and \code{\link{reply}} for use with contexts.
 #' @examples
 #' s <- socket("req", listen = "inproc://nanonext")
@@ -64,16 +64,16 @@ context <- function(socket) .Call(rnng_ctx_open, socket)
 #' Technical Utility: Open Context
 #'
 #' Open a new Context to be used with a Socket. This function is a performance
-#'     variant of \code{\link{context}}, designed to wrap a socket in a function
-#'     argument when calling \code{\link{request}} or \code{\link{reply}}.
+#' variant of \code{\link{context}}, designed to wrap a socket in a function
+#' argument when calling \code{\link{request}} or \code{\link{reply}}.
+#'
+#' External pointers created by this function are unclassed, hence methods for
+#' contexts such as \code{\link{close}} will not work (use \code{\link{reap}}
+#' instead). Otherwise they function identically to a Context when passed to all
+#' messaging functions.
 #'
 #' @param socket a Socket.
 #'
-#' @details External pointers created by this function are unclassed, hence
-#'     methods for contexts such as \code{\link{close}} will not work (use
-#'     \code{\link{reap}} instead). Otherwise they function identically to a
-#'     Context when passed to all messaging functions.
-#'
 #' @return An external pointer.
 #'
 #' @export
@@ -89,44 +89,45 @@ close.nanoContext <- function(con, ...) invisible(.Call(rnng_ctx_close, con))
 #' Reply over Context (RPC Server for Req/Rep Protocol)
 #'
 #' Implements an executor/server for the rep node of the req/rep protocol.
-#'     Awaits data, applies an arbitrary specified function, and returns the
-#'     result to the caller/client.
+#' Awaits data, applies an arbitrary specified function, and returns the result
+#' to the caller/client.
+#'
+#' Receive will block while awaiting a message to arrive and is usually the
+#' desired behaviour. Set a timeout to allow the function to return if no data
+#' is forthcoming.
+#'
+#' In the event of an error in either processing the messages or in evaluation
+#' of the function with respect to the data, a nul byte \code{00} (or serialized
+#' nul byte) will be sent in reply to the client to signal an error. This is to
+#' be distinguishable from a possible return value. \code{\link{is_nul_byte}}
+#' can be used to test for a nul byte.
 #'
 #' @param context a Context.
 #' @param execute a function which takes the received (converted) data as its
-#'     first argument. Can be an anonymous function of the form
-#'     \code{function(x) do(x)}. Additional arguments can also be passed in
-#'     through \sQuote{...}.
+#'   first argument. Can be an anonymous function of the form
+#'   \code{function(x) do(x)}. Additional arguments can also be passed in
+#'   through \sQuote{...}.
 #' @param send_mode [default 'serial'] character value or integer equivalent -
-#'     either \sQuote{serial} (1L) to send serialised R objects, or \sQuote{raw}
-#'     (2L) to send atomic vectors of any type as a raw byte vector.
+#'   either \sQuote{serial} (1L) to send serialised R objects, or \sQuote{raw}
+#'   (2L) to send atomic vectors of any type as a raw byte vector.
 #' @param recv_mode [default 'serial'] character value or integer equivalent -
-#'     one of \sQuote{serial} (1L), \sQuote{character} (2L), \sQuote{complex}
-#'     (3L), \sQuote{double} (4L), \sQuote{integer} (5L), \sQuote{logical} (6L),
-#'     \sQuote{numeric} (7L), \sQuote{raw} (8L), or \sQuote{string} (9L). The
-#'     default \sQuote{serial} means a serialised R object; for the other
-#'     modes, received bytes are converted into the respective mode.
-#'     \sQuote{string} is a faster option for length one character vectors.
+#'   one of \sQuote{serial} (1L), \sQuote{character} (2L), \sQuote{complex}
+#'   (3L), \sQuote{double} (4L), \sQuote{integer} (5L), \sQuote{logical} (6L),
+#'   \sQuote{numeric} (7L), \sQuote{raw} (8L), or \sQuote{string} (9L). The
+#'   default \sQuote{serial} means a serialised R object; for the other modes,
+#'   received bytes are converted into the respective mode. \sQuote{string} is a
+#'   faster option for length one character vectors.
 #' @param timeout [default NULL] integer value in milliseconds or NULL, which
-#'     applies a socket-specific default, usually the same as no timeout. Note
-#'     that this applies to receiving the request. The total elapsed time would
-#'     also include performing 'execute' on the received data. The timeout then
-#'     also applies to sending the result (in the event that the requestor has
-#'     become unavailable since sending the request).
-#' @param ... additional arguments passed to the function specified by 'execute'.
+#'   applies a socket-specific default, usually the same as no timeout. Note
+#'   that this applies to receiving the request. The total elapsed time would
+#'   also include performing 'execute' on the received data. The timeout then
+#'   also applies to sending the result (in the event that the requestor has
+#'   become unavailable since sending the request).
+#' @param ... additional arguments passed to the function specified by
+#'   'execute'.
 #'
 #' @return Integer exit code (zero on success).
 #'
-#' @details Receive will block while awaiting a message to arrive and is usually
-#'     the desired behaviour. Set a timeout to allow the function to return
-#'     if no data is forthcoming.
-#'
-#'     In the event of an error in either processing the messages or in
-#'     evaluation of the function with respect to the data, a nul byte \code{00}
-#'     (or serialized nul byte) will be sent in reply to the client to signal an
-#'     error. This is to be distinguishable from a possible return value.
-#'     \code{\link{is_nul_byte}} can be used to test for a nul byte.
-#'
 #' @inheritSection send Send Modes
 #'
 #' @examples
@@ -160,9 +161,7 @@ reply <- function(context,
   block <- if (is.null(timeout)) TRUE else timeout
   res <- recv(context, mode = recv_mode, block = block)
   is_error_value(res) && return(res)
-  on.exit(expr = send(context, data = as.raw(0L), mode = send_mode, block = TRUE))
-  data <- execute(res, ...)
-  on.exit()
+  data <- .Call(rnng_eval_safe, as.call(list(execute, res, ...)))
   send(context, data = data, mode = send_mode, block = block)
 
 }
@@ -170,41 +169,39 @@ reply <- function(context,
 #' Request over Context (RPC Client for Req/Rep Protocol)
 #'
 #' Implements a caller/client for the req node of the req/rep protocol. Sends
-#'     data to the rep node (executor/server) and returns an Aio, which can be
-#'     called for the value when required.
+#' data to the rep node (executor/server) and returns an Aio, which can be
+#' called for the value when required.
+#'
+#' Sending the request and receiving the result are both performed async, hence
+#' the function will return immediately with a \sQuote{recvAio} object. Access
+#' the return value at \code{$data}.
+#'
+#' This is designed so that the process on the server can run concurrently
+#' without blocking the client.
+#'
+#' Optionally use \code{\link{call_aio}} on the \sQuote{recvAio} to call (and
+#' wait for) the result.
+#'
+#' If an error occured in the server process, a nul byte \code{00} will be
+#' received. This allows an error to be easily distinguished from a NULL return
+#' value. \code{\link{is_nul_byte}} can be used to test for a nul byte.
+#'
+#' It is recommended to use a new context for each request to ensure consistent
+#' state tracking. For safety, the context used for the request is closed when
+#' all references to the returned \sQuote{recvAio} are removed and the object is
+#' garbage collected.
 #'
 #' @inheritParams reply
 #' @inheritParams recv
 #' @param data an object (if send_mode = \sQuote{raw}, a vector).
 #' @param timeout [default NULL] integer value in milliseconds or NULL, which
-#'     applies a socket-specific default, usually the same as no timeout.
+#'   applies a socket-specific default, usually the same as no timeout.
 #' @param cv (optional) a \sQuote{conditionVariable} to signal when the async
-#'     receive is complete, or NULL. If any other value is supplied, this will
-#'     cause the pipe connection to be dropped when the async receive is
-#'     complete.
+#'   receive is complete, or NULL. If any other value is supplied, this will
+#'   cause the pipe connection to be dropped when the async receive is complete.
 #'
 #' @return A \sQuote{recvAio} (object of class \sQuote{mirai} and
-#'     \sQuote{recvAio}) (invisibly).
-#'
-#' @details Sending the request and receiving the result are both performed
-#'     async, hence the function will return immediately with a \sQuote{recvAio}
-#'     object. Access the return value at \code{$data}.
-#'
-#'     This is designed so that the process on the server can run concurrently
-#'     without blocking the client.
-#'
-#'     Optionally use \code{\link{call_aio}} on the \sQuote{recvAio} to call
-#'     (and wait for) the result.
-#'
-#'     If an error occured in the server process, a nul byte \code{00} will be
-#'     received. This allows an error to be easily distinguished from a NULL
-#'     return value. \code{\link{is_nul_byte}} can be used to test for a nul
-#'     byte.
-#'
-#'     It is recommended to use a new context for each request to ensure
-#'     consistent state tracking. For safety, the context used for the request
-#'     is closed when all references to the returned \sQuote{recvAio} are
-#'     removed and the object is garbage collected.
+#'   \sQuote{recvAio}) (invisibly).
 #'
 #' @inheritSection send Send Modes
 #' @inheritSection recv_aio Signalling
@@ -251,22 +248,3 @@ request <- function(context,
                     timeout = NULL,
                     cv = NULL)
   data <- .Call(rnng_request, context, data, send_mode, recv_mode, timeout, cv, environment())
-
-#' Set Promise Context
-#'
-#' If called from an appropriate context, creates an event-driven promise that
-#'     will resolve asynchronously when the request is complete.
-#'
-#' @param x a 'recvAio' object returned by \code{\link{request}}.
-#' @param ctx the context environment.
-#'
-#' @details The object passed as \sQuote{x} is returned regardless of whether
-#'     the promise context was set successfully or not. If successful,
-#'     \sQuote{x} is modified in place with the promise context.
-#'
-#' @return The object \sQuote{x}.
-#'
-#' @keywords internal
-#' @export
-#'
-set_promise_context <- function(x, ctx) .Call(rnng_set_promise_context, x, ctx)
