@@ -540,6 +540,17 @@ test_true(recv(disp, block = 500L))
 test_zero(send(disp, NULL, block = 500L))
 test_null(recv(s, block = 500L))
 test_zero(close(s))
+test_class("nanoSocket", s <- socket(protocol = "rep"))
+test_zero(pipe_notify(s, cv, add = TRUE))
+test_zero(dial(s, url = "inproc://disp/1"))
+test_true(wait(cv))
+test_zero(send(disp, TRUE, block = 500L))
+test_true(recv(s, block = 500L))
+test_true(.mark())
+test_zero(send(s, NULL, block = 500L))
+test_true(!.mark(FALSE))
+test_null(recv(disp, block = 500L))
+test_zero(close(s))
 rm(disp)
 
 test_equal(nanonext:::.DollarNames.ncurlAio(NULL, "sta"), "status")
@@ -651,9 +662,11 @@ if (promises) test_zero(close(s1))
 if (promises) test_zero(close(s))
 if (promises) later::run_now()
 test_true(!identical(get0(".Random.seed"), {.advance(); .Random.seed}))
+test_type("integer", .Call(nanonext:::rnng_traverse_precious))
 if (Sys.info()[["sysname"]] == "Linux") {
   rm(list = ls())
   gc()
+  .Call(nanonext:::rnng_thread_shutdown)
   Sys.sleep(1L)
   .Call(nanonext:::rnng_fini)
   invisible()
